==> next-cli-1.2.1.tgz: package/src/commands/ci.js <==
const { Command, flags } = require("@oclif/command");
const { Deployer } = require("@dotdev/next-deployer");
const { Config, State } = require("@dotdev/next-config");

class CiCommand extends Command {
  async run() {
    const flags = this.parse(CiCommand).flags;

    const config = Config.resolve();

    const state = State.resolve();

    if (flags["output-theme-ids"]) {
      this.outputThemeIds(state);
      return;
    }

    if (flags["env"]) {
      await state.clear();
    }

    await this.environment(config, flags.env);

    await this.credentials(config, flags["api-key"], flags.password);

    const themes = await this.theme(
      config,
      state,
      flags.theme,
      flags.tag,
      flags.create
    );

    if (flags["delete"]) {
      await this.deleteThemes(config, themes);
      return;
    }

    await state.set("targetThemes", themes);
  }

  async environment(config, env) {
    const environments = config.get("environments");

    if (!environments) {
      this.error(
        "No environments defined, check your project next.config.json file"
      );
    }

    if (config.hasEnvironment() && !env) return;

    const handles = Object.keys(environments);

    if (1 == handles.length) {
      await config.setEnvironment(handles.shift());
      return;
    }

    if (!config.hasEnvironment() && !env) {
      this.error("Environment must be selected with the --env flag");
    }

    if (!handles.includes(env)) {
      this.error(`Environment ${env} is not configured for this project`);
    }

    await config.setEnvironment(env);
  }

  async credentials(config, apiKey, password) {
    if (config.hasCredentials() && (!apiKey || !password)) return;

    if (!config.hasCredentials() && (!apiKey || !password)) {
      this.error(
        "Credentials must be provided with the --apiKey and --password flags"
      );
    }

    await config.setCredentials(apiKey, password);
  }

  async theme(config, state, themeId, themeTag, themeCreate) {
    if (config.hasTheme() && !themeId && !themeTag) return [config.getTheme()];

    if (!config.hasTheme() && !themeId && !themeTag) {
      this.error("Theme must be selected using either --theme or --tag flags");
    }

    const deployer = await new Deployer(config);

    this.log(
      `Fetching themes for environment ${config.getEnvironment().handle}...`
    );

    const themes = await deployer.getThemes();

    if (themeId) {
      const selectedTheme = themes.find(
        theme => themeId === theme.id.toString()
      );

      if (!selectedTheme) {
        this.error(`Theme with ID ${themeId} could not be found`);
      }

      await config.setTheme(selectedTheme);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(
          await deployer.getAssetBase(selectedTheme.id)
        );
      }

      this.log(`Connected theme ID ${themeId}`);

      return [config.getTheme()];
    }

    const normalisedTag = themeTag.toLowerCase();

    const taggedThemes = themes.filter(theme => {
      const normalisedThemeName = theme.name.toLowerCase();

      return (
        normalisedTag === normalisedThemeName ||
        normalisedThemeName.includes(`[${normalisedTag}]`)
      );
    });

    this.log(
      `Resolved ${taggedThemes.length} theme${
        1 === taggedThemes.length ? "" : "s"
      } with tag ${normalisedTag}`
    );

    if (0 < taggedThemes.length) {
      this.log(`Themes: ${taggedThemes.map(theme => theme.name).join(", ")}`);

      await config.setTheme(taggedThemes[0]);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(
          await deployer.getAssetBase(taggedThemes[0].id)
        );
      }
    } else if (themeCreate) {
      this.log(`Creating new theme for tag ${normalisedTag}...`);

      const theme = await deployer.createTheme(themeTag);

      await config.setTheme(theme);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(await deployer.getAssetBase(theme.id));
      }

      taggedThemes.push(theme);

      await state.push("createdThemes", theme);

      this.log(`Theme created, ready for deployment`);
    }

    return taggedThemes;
  }

  outputThemeIds(state) {
    this.log(
      state
        .get("targetThemes", [])
        .map(theme => theme.id)
        .join(",")
    );
  }

  async deleteThemes(config, themes) {
    const deployer = await new Deployer(config);

    this.log(`Deleting matched themes...`);

    await Promise.all(
      themes
        .filter(theme => "main" !== theme.role)
        .map(theme => deployer.deleteTheme(theme.id))
    );

    this.log(`Themes deleted`);
  }
}

CiCommand.description = "connect to an environment without interaction";

CiCommand.flags = {
  env: flags.string({ description: "environment to connect" }),
  "api-key": flags.string({ description: "shopify api key" }),
  password: flags.string({ description: "shopify password" }),
  tag: flags.string({ description: "theme tag to connect" }),
  theme: flags.string({ description: "theme id to connect" }),
  "output-theme-ids": flags.boolean({
    description: "output resolved theme ids as a comma separated list"
  }),
  create: flags.boolean({ description: "create theme if not exists" }),
  delete: flags.boolean({ description: "delete matching themes" })
};

module.exports = CiCommand;

==> next-cli-2.1.1.tgz: package/src/commands/ci.js <==
const { Command, flags } = require("@oclif/command");
const { Deployer } = require("@dotdev/next-deployer");
const { Config, State } = require("@dotdev/next-config");

class CiCommand extends Command {
  async run() {
    const flags = this.parse(CiCommand).flags;

    const config = Config.resolve();

    const state = State.resolve();

    if (flags["output-theme-ids"]) {
      this.outputThemeIds(state);
      return;
    }

    if (flags["env"]) {
      await state.clear();
    }

    await this.environment(config, flags.env);

    await this.credentials(config, flags["api-key"], flags.password);

    const themes = await this.theme(
      config,
      state,
      flags.theme,
      flags.tag,
      flags.create,
      flags["live-theme"]
    );

    if (flags["delete"]) {
      await this.deleteThemes(config, themes);
      return;
    }

    await state.set("targetThemes", themes);
  }

  async environment(config, env) {
    const environments = config.get("environments");

    if (!environments) {
      this.error(
        "No environments defined, check your project next.config.json file"
      );
    }

    if (config.hasEnvironment() && !env) return;

    const handles = Object.keys(environments);

    if (1 == handles.length) {
      await config.setEnvironment(handles.shift());
      return;
    }

    if (!config.hasEnvironment() && !env) {
      this.error("Environment must be selected with the --env flag");
    }

    if (!handles.includes(env)) {
      this.error(`Environment ${env} is not configured for this project`);
    }

    await config.setEnvironment(env);
  }

  async credentials(config, apiKey, password) {
    if (config.hasCredentials() && (!apiKey || !password)) return;

    if (!config.hasCredentials() && (!apiKey || !password)) {
      this.error(
        "Credentials must be provided with the --apiKey and --password flags"
      );
    }

    await config.setCredentials(apiKey, password);
  }

  async theme(config, state, themeId, themeTag, themeCreate, includeLiveTheme) {
    if (config.hasTheme() && !themeId && !themeTag && !includeLiveTheme)
      return [config.getTheme()];

    if (!config.hasTheme() && !themeId && !themeTag && !includeLiveTheme) {
      this.error(
        "Theme must be selected using either --theme, --tag or --live-theme flags"
      );
    }

    const deployer = await new Deployer(config);

    this.log(
      `Fetching themes for environment ${config.getEnvironment().handle}...`
    );

    const themes = await deployer.getThemes();

    if (themeId) {
      const selectedTheme = themes.find(
        (theme) => themeId === theme.id.toString()
      );

      if (!selectedTheme) {
        this.error(`Theme with ID ${themeId} could not be found`);
      }

      await config.setTheme(selectedTheme);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(
          await deployer.getAssetBase(selectedTheme.id)
        );
      }

      this.log(`Connected theme ID ${themeId}`);

      return [config.getTheme()];
    }

    let taggedThemes = [];
    const normalisedTag = themeTag ? themeTag.toLowerCase() : undefined;

    // search for themes matching tag if --tag flag provided
    if (normalisedTag) {
      taggedThemes = themes.filter((theme) => {
        const normalisedThemeName = theme.name.toLowerCase();

        return (
          normalisedTag === normalisedThemeName ||
          normalisedThemeName.includes(`[${normalisedTag}]`)
        );
      });

      this.log(
        `Resolved ${taggedThemes.length} theme${
          1 === taggedThemes.length ? "" : "s"
        } with tag ${normalisedTag}`
      );
    }

    // append live theme if the --live-theme flag provided, even if the tag did not match
    if (includeLiveTheme) {
      const liveTheme = themes.find((theme) => "main" === theme.role);
      if (!taggedThemes.find((theme) => theme.id === liveTheme.id)) {
        this.log(`Resolved live theme: ${liveTheme.name}`);
        taggedThemes.push(liveTheme);
      }
    }

    if (0 < taggedThemes.length) {
      this.log(`Themes: ${taggedThemes.map((theme) => theme.name).join(", ")}`);

      await config.setTheme(taggedThemes[0]);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(
          await deployer.getAssetBase(taggedThemes[0].id)
        );
      }
    } else if (themeCreate) {
      if (!normalisedTag) {
        throw new Error(
          "theme cannot be created without providing a --tag argument"
        );
      }

      this.log(`Creating new theme for tag ${normalisedTag}...`);

      const theme = await deployer.createTheme(themeTag);

      await config.setTheme(theme);

      if (!config.hasAssetBase()) {
        await config.setAssetBase(await deployer.getAssetBase(theme.id));
      }

      taggedThemes.push(theme);

      await state.push("createdThemes", theme);

      this.log(`Theme created, ready for deployment`);
    }

    return taggedThemes;
  }

  outputThemeIds(state) {
    this.log(
      state
        .get("targetThemes", [])
        .map((theme) => theme.id)
        .join(",")
    );
  }

  async deleteThemes(config, themes) {
    const deployer = await new Deployer(config);

    this.log(`Deleting matched themes...`);

    await Promise.all(
      themes
        .filter((theme) => "main" !== theme.role)
        .map((theme) => deployer.deleteTheme(theme.id))
    );

    this.log(`Themes deleted`);
  }
}

CiCommand.description = "connect to an environment without interaction";

CiCommand.flags = {
  env: flags.string({ description: "environment to connect" }),
  "api-key": flags.string({ description: "shopify api key" }),
  password: flags.string({ description: "shopify password" }),
  tag: flags.string({ description: "theme tag to connect" }),
  theme: flags.string({ description: "theme id to connect" }),
  "output-theme-ids": flags.boolean({
    description: "output resolved theme ids as a comma separated list",
  }),
  create: flags.boolean({ description: "create theme if not exists" }),
  delete: flags.boolean({ description: "delete matching themes" }),
  "live-theme": flags.boolean({
    description: "include live theme in matches",
  }),
};

module.exports = CiCommand;
